//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by vba.rc
//
#define IDS_UNSUPPORTED_VBA_SGM         1
#define IDS_CANNOT_LOAD_SGM             2
#define IDS_SAVE_GAME_NOT_USING_BIOS    3
#define IDC_DEFAULTS                    3
#define IDS_SAVE_GAME_USING_BIOS        4
#define IDS_UNSUPPORTED_SAVE_TYPE       5
#define IDS_CANNOT_OPEN_FILE            6
#define IDS_BAD_ZIP_FILE                7
#define IDS_NO_IMAGE_ON_ZIP             8
#define IDS_ERROR_OPENING_IMAGE         9
#define IDS_ERROR_READING_IMAGE         10
#define IDS_UNSUPPORTED_BIOS_FUNCTION   11
#define IDS_INVALID_BIOS_FILE_SIZE      12
#define IDS_INVALID_CHEAT_CODE          13
#define IDS_UNKNOWN_ARM_OPCDOE          14
#define IDS_UNKNOWN_THUMB_OPCODE        15
#define IDS_ERROR_CREATING_FILE         16
#define IDS_FAILED_TO_READ_SGM          17
#define IDS_FAILED_TO_READ_RTC          18
#define IDS_UNSUPPORTED_VB_SGM          19
#define IDS_CANNOT_LOAD_SGM_FOR         20
#define IDS_ERROR_OPENING_IMAGE_FROM    21
#define IDS_ERROR_READING_IMAGE_FROM    22
#define IDS_UNSUPPORTED_ROM_SIZE        23
#define IDS_UNSUPPORTED_RAM_SIZE        24
#define IDS_UNKNOWN_CARTRIDGE_TYPE      25
#define IDS_MAXIMUM_NUMBER_OF_CHEATS    26
#define IDS_INVALID_GAMESHARK_CODE      27
#define IDS_INVALID_GAMEGENIE_CODE      28
#define IDS_INVALID_CHEAT_TO_REMOVE     29
#define IDS_INVALID_CHEAT_CODE_ADDRESS  30
#define IDS_UNSUPPORTED_CHEAT_LIST_VERSION 31
#define IDS_UNSUPPORTED_CHEAT_LIST_TYPE 32
#define IDS_INVALID_GSA_CODE            33
#define IDS_CANNOT_IMPORT_SNAPSHOT_FOR  34
#define IDS_UNSUPPORTED_SNAPSHOT_FILE   35
#define IDS_UNSUPPORTED_ARM_MODE        36
#define IDS_UNSUPPORTED_CODE_FILE       37
#define IDS_GSA_CODE_WARNING            38
#define IDS_INVALID_CBA_CODE            39
#define IDS_CBA_CODE_WARNING            40
#define IDS_OUT_OF_MEMORY               41
#define IDS_WRONG_GAMESHARK_CODE        42
#define IDS_UNSUPPORTED_GAMESHARK_CODE  43
#define IDD_REGISTERS                   102
#define IDD_DEBUG                       103
#define IDR_MENU                        104
#define IDD_ABOUT                       105
#define IDR_ACCELERATOR                 106
#define IDD_CHEATS                      107
#define IDD_ADD_CHEAT                   108
#define IDD_DIRECTORIES                 109
#define IDD_CONFIG                      110
#define IDD_GS                          111
#define IDD_GG                          112
#define IDD_CHEAT_LIST                  113
#define IDD_ASSOCIATIONS                114
#define IDR_GB_PRINTER                  115
#define IDD_GBA_ROM_INFO                116
#define IDD_GB_ROM_INFO                 117
#define IDD_GB_CHEAT_LIST               118
#define IDD_ADD_CHEAT_DLG               119
#define IDD_GB_PRINTER                  120
#define IDD_MOTION_CONFIG               121
#define IDD_LANG_SELECT                 122
#define IDD_CODE_SELECT                 123
#define IDD_OPENDLG                     124
#define IDD_MAP_VIEW                    126
#define IDD_PALETTE_VIEW                127
#define IDD_MEM_VIEWER                  128
#define IDD_OAM_VIEW                    130
#define IDD_ACCEL_EDITOR                131
#define IDD_TILE_VIEWER                 132
#define IDD_GB_COLORS                   133
#define IDD_DISASSEMBLE                 134
#define IDD_GDB_PORT                    135
#define IDD_GDB_WAITING                 136
#define IDD_LOGGING                     137
#define IDD_EXPORT_SPS                  138
#define IDD_ADDR_SIZE                   139
#define IDD_MODES                       140
#define IDD_DRIVERS                     142
#define IDD_THROTTLE                    143
#define IDD_GB_DISASSEMBLE              144
#define IDD_GB_OAM_VIEW                 145
#define IDD_GB_TILE_VIEWER              146
#define IDD_GB_MAP_VIEW                 147
#define IDD_GB_PALETTE_VIEW             148
#define IDD_MODE_CONFIRM                149
#define IDD_REWIND_INTERVAL             150
#define IDD_IO_VIEWER                   151
#define IDD_MAX_SCALE                   154
#define IDD_BUG_REPORT                  155
#define IDD_GAME_OVERRIDES              156
#define IDC_R0                          1000
#define IDC_EDIT_UP                     1000
#define IDC_R1                          1001
#define IDC_EDIT_DOWN                   1001
#define IDC_R2                          1002
#define IDC_EDIT_LEFT                   1002
#define IDC_R3                          1003
#define IDC_EDIT_RIGHT                  1003
#define IDC_R4                          1004
#define IDC_EDIT_BUTTON_A               1004
#define IDC_R5                          1005
#define IDC_EDIT_BUTTON_B               1005
#define IDC_R6                          1006
#define IDC_EDIT_BUTTON_SELECT          1006
#define IDC_R7                          1007
#define IDC_EDIT_BUTTON_START           1007
#define IDC_R8                          1008
#define ID_OK                           1008
#define IDC_R9                          1009
#define ID_CANCEL                       1009
#define ID_SAVE                         1009
#define IDC_R10                         1010
#define IDC_EDIT_SPEED                  1010
#define IDC_R11                         1011
#define IDC_EDIT_CAPTURE                1011
#define IDC_R12                         1012
#define IDC_EDIT_BUTTON_L               1012
#define IDC_R13                         1013
#define IDC_EDIT_BUTTON_GS              1013
#define IDC_R14                         1014
#define IDC_EDIT_BUTTON_R               1014
#define IDC_R15                         1015
#define IDC_R16                         1016
#define IDC_R17                         1017
#define IDC_N_FLAG                      1018
#define IDC_ROM_DIR                     1018
#define IDC_Z_FLAG                      1019
#define IDC_NEXT                        1019
#define IDC_BATTERY_DIR                 1019
#define IDC_C_FLAG                      1020
#define IDC_CONTINUE                    1020
#define IDC_SAVE_DIR                    1020
#define IDC_V_FLAG                      1021
#define IDC_CAPTURE_DIR                 1021
#define IDC_CHEAT_LIST                  1021
#define IDC_IRQ                         1022
#define IDC_ROM_PATH                    1022
#define IDC_START                       1022
#define IDC_T_FLAG                      1023
#define IDC_BATTERY_PATH                1023
#define IDC_SEARCH                      1023
#define IDS_DIRECTX_7_REQUIRED          1024
#define IDC_SAVE_PATH                   1024
#define IDC_ADD_CHEAT                   1024
#define IDC_M4                          1025
#define IDC_CAPTURE_PATH                1025
#define IDC_OLD_VALUE                   1025
#define IDC_ADD_GS_CHEAT                1025
#define IDS_DISABLING_VIDEO_MEMORY      1025
#define IDC_ADD_GAMESHARK               1025
#define IDC_M3                          1026
#define IDC_SPECIFIC_VALUE              1026
#define IDS_SETTING_WILL_BE_EFFECTIVE   1026
#define IDC_GBROM_DIR                   1026
#define IDC_M2                          1027
#define IDS_DISABLING_EMULATION_ONLY    1027
#define IDC_GBROM_PATH                  1027
#define IDC_M1                          1028
#define IDC_SIZE_8                      1028
#define IDS_FAILED_TO_OPEN_FILE         1028
#define IDC_ROM_DIR_RESET               1028
#define IDC_M0                          1029
#define IDC_SIZE_16                     1029
#define IDS_FAILED_TO_READ_ZIP_DIR      1029
#define IDC_GBROM_DIR_RESET             1029
#define IDC_SIZE_32                     1030
#define IDS_UNSUPPORTED_FILE_TYPE       1030
#define IDC_BATTERY_DIR_RESET           1030
#define IDC_EQ                          1031
#define IDS_CANNOT_CREATE_DIRECTSOUND   1031
#define IDC_SAVE_DIR_RESET              1031
#define IDC_NE                          1032
#define IDS_CANNOT_SETCOOPERATIVELEVEL  1032
#define IDC_CAPTURE_DIR_RESET           1032
#define IDC_LT                          1033
#define IDS_CANNOT_CREATESOUNDBUFFER    1033
#define IDC_LE                          1034
#define IDS_CANNOT_SETFORMAT_PRIMARY    1034
#define IDC_GT                          1035
#define IDS_CANNOT_CREATESOUNDBUFFER_SEC 1035
#define IDC_GE                          1036
#define IDS_CANNOT_PLAY_PRIMARY         1036
#define IDC_SIGNED                      1037
#define IDS_SEARCH_PRODUCED_TOO_MANY    1037
#define IDC_UNSIGNED                    1038
#define IDS_NUMBER_CANNOT_BE_EMPTY      1038
#define IDS_INVALID_ADDRESS             1039
#define IDC_HEXADECIMAL                 1040
#define IDS_MISALIGNED_HALFWORD         1040
#define IDC_VALUE                       1041
#define IDS_MISALIGNED_WORD             1041
#define IDC_ADDRESS                     1042
#define IDS_VALUE_CANNOT_BE_EMPTY       1042
#define IDS_ERROR_ON_STARTDOC           1043
#define IDC_R                           1043
#define IDS_ERROR_ON_STARTPAGE          1044
#define IDC_G                           1044
#define IDS_ERROR_PRINTING_ON_STRETCH   1045
#define IDC_B                           1045
#define IDC_UPDATE                      1046
#define IDS_ERROR_ON_ENDPAGE            1046
#define IDC_TILE_NUM                    1046
#define IDC_GGDESC                      1047
#define IDS_ERROR_ON_ENDDOC             1047
#define IDC_FLIP                        1047
#define IDC_GGCODE                      1048
#define IDS_ERROR                       1048
#define IDC_PALETTE_NUM                 1048
#define IDC_GGADD                       1049
#define IDS_JOY_LEFT                    1049
#define IDC_GGDEL                       1050
#define IDS_JOY_RIGHT                   1050
#define IDC_GGLIST                      1051
#define IDS_JOY_UP                      1051
#define IDC_GGRES                       1052
#define IDS_JOY_DOWN                    1052
#define IDC_GGQUIT                      1053
#define IDS_JOY_BUTTON                  1053
#define IDC_GSDESC                      1054
#define IDS_SELECT_ROM_DIR              1054
#define IDC_GSCODE                      1055
#define IDS_SELECT_BATTERY_DIR          1055
#define IDC_GSADD                       1056
#define IDS_SELECT_SAVE_DIR             1056
#define IDC_GSDEL                       1057
#define IDS_SELECT_CAPTURE_DIR          1057
#define IDC_GSLIST                      1058
#define IDS_SELECT_BIOS_FILE            1058
#define IDC_GSRES                       1059
#define IDS_RESET                       1059
#define IDC_GSQUIT                      1060
#define IDS_AUTOFIRE_A_DISABLED         1060
#define IDC_FREEZE                      1061
#define IDS_AUTOFIRE_A                  1061
#define IDS_AUTOFIRE_B_DISABLED         1062
#define IDS_AUTOFIRE_B                  1063
#define IDS_AUTOFIRE_L_DISABLED         1064
#define IDS_AUTOFIRE_L                  1065
#define IDS_AUTOFIRE_R_DISABLED         1066
#define IDC_REMOVE                      1067
#define IDS_AUTOFIRE_R                  1067
#define IDC_REMOVE_ALL                  1068
#define IDS_SELECT_ROM                  1068
#define IDS_SELECT_SAVE_GAME_NAME       1069
#define IDC_ENABLE                      1070
#define IDS_LOADED_STATE                1070
#define IDS_LOADED_STATE_N              1071
#define IDS_WROTE_STATE                 1072
#define IDS_WROTE_STATE_N               1073
#define IDC_RESTORE                     1074
#define IDS_LOADED_BATTERY              1074
#define IDC_GBA                         1075
#define IDS_SELECT_CAPTURE_NAME         1075
#define IDC_AGB                         1076
#define IDS_SCREEN_CAPTURE              1076
#define IDC_BIN                         1077
#define IDS_ADDRESS                     1077
#define IDC_GB                          1078
#define IDS_OLD_VALUE                   1078
#define IDC_SGB                         1079
#define IDC_ROM_TITLE                   1079
#define IDS_NEW_VALUE                   1079
#define IDC_CGB                         1080
#define IDC_ROM_GAME_CODE               1080
#define IDS_ADD_CHEAT_CODE              1080
#define IDC_GBC                         1081
#define IDC_ROM_MAKER_CODE              1081
#define IDS_CODE                        1081
#define IDC_ROM_UNIT_CODE               1082
#define IDS_DESCRIPTION                 1082
#define IDC_ROM_DEVICE_TYPE             1083
#define IDS_STATUS                      1083
#define IDC_ROM_VERSION                 1084
#define IDS_ADD_GG_CODE                 1084
#define IDC_ROM_CRC                     1085
#define IDS_ADD_GS_CODE                 1085
#define IDC_ROM_COLOR                   1086
#define IDC_CODE                        1086
#define IDS_POCKET_PRINTER              1086
#define IDC_ROM_MAKER_NAME              1086
#define IDC_ROM_SIZE                    1087
#define IDC_DESC                        1087
#define IDS_UNKNOWN                     1087
#define IDC_ROM_RAM_SIZE                1088
#define IDC_ADD_GG_CHEAT                1088
#define IDS_NONE                        1088
#define IDC_ROM_DEST_CODE               1089
#define IDC_GB_PRINTER                  1089
#define IDS_FAILED_TO_LOAD_LIBRARY      1089
#define IDC_ROM_LIC_CODE                1090
#define IDC_1X                          1090
#define IDS_FAILED_TO_GET_LOCINFO       1090
#define IDC_ROM_CHECKSUM                1091
#define IDC_2X                          1091
#define IDS_SELECT_CHEAT_LIST_NAME      1091
#define IDC_3X                          1092
#define IDS_FILTER_BIOS                 1092
#define IDC_4X                          1093
#define IDS_FILTER_ROM                  1093
#define IDC_ROM_MAKER_NAME2             1093
#define ID_PRINT                        1094
#define IDS_FILTER_SGM                  1094
#define IDC_ADD_GSA                     1095
#define IDC_ADD_CODE                    1095
#define IDS_FILTER_CHEAT_LIST           1095
#define IDC_TRANSLATION_BY              1096
#define IDS_FILTER_PNG                  1096
#define IDC_LANG_STRING                 1097
#define IDS_LOADED_CHEATS               1097
#define IDC_LANG_NAME                   1098
#define IDS_ERROR_DISP_COLOR            1098
#define IDS_ADD_GSA_CODE                1099
#define IDC_GAME_LIST                   1099
#define IDS_FILTER_SPS                  1100
#define IDS_SELECT_SNAPSHOT_FILE        1101
#define IDC_ADD_CODEBREAKER             1101
#define IDS_FILTER_SAV                  1102
#define IDS_SELECT_BATTERY_FILE         1103
#define IDS_FILTER_GBS                  1104
#define IDS_FILTER_GCF                  1105
#define IDS_SELECT_CODE_FILE            1106
#define IDS_SAVE_WILL_BE_LOST           1107
#define IDS_CONFIRM_ACTION              1108
#define IDS_CODES_WILL_BE_LOST          1109
#define IDS_FILTER_SPC                  1110
#define IDS_ADD_CBA_CODE                1111
#define IDS_FILTER_WAV                  1112
#define IDS_SELECT_WAV_NAME             1113
#define IDC_FRAME_0                     1113
#define IDS_FILTER_GBROM                1114
#define IDC_FRAME_1                     1114
#define IDC_BG0                         1115
#define IDS_FILTER_PAL                  1115
#define IDC_BG1                         1116
#define IDS_SELECT_PALETTE_NAME         1116
#define IDC_BG2                         1117
#define IDS_SEARCH_PRODUCED_NO_RESULTS  1117
#define IDC_BG3                         1118
#define IDS_ERROR_BINDING               1118
#define IDS_ERROR_LISTENING             1119
#define IDS_ERROR_CREATING_SOCKET       1120
#define IDS_ACK_NOT_RECEIVED            1121
#define IDS_ERROR_NOT_GBA_IMAGE         1122
#define IDS_EEPROM_NOT_SUPPORTED        1123
#define IDC_MAP_VIEW                    1124
#define IDS_FILTER_DUMP                 1124
#define IDC_PALETTE_VIEW                1125
#define IDS_SELECT_DUMP_FILE            1125
#define IDC_PALETTE_VIEW_OBJ            1126
#define IDC_REFRESH                     1126
#define IDS_FILTER_AVI                  1126
#define IDC_SAVE                        1127
#define IDC_GOPC                        1127
#define IDS_SELECT_AVI_NAME             1127
#define IDC_APPLY                       1127
#define IDS_INVALID_THROTTLE_VALUE      1128
#define IDC_REFRESH2                    1129
#define IDS_FILTER_INI                  1129
#define IDS_SELECT_SKIN_FILE            1130
#define IDC_CLOSE                       1131
#define IDS_FILTER_VMV                  1131
#define IDS_SELECT_MOVIE_NAME           1132
#define IDS_BUG_REPORT                  1133
#define IDS_UNSUPPORTED_MOVIE_VERSION   1134
#define IDS_END_OF_MOVIE                1135
#define IDC_COLOR                       1136
#define IDS_INVALID_INTERVAL_VALUE      1136
#define IDC_SAVE_BG                     1137
#define IDS_REGISTRY                    1137
#define IDC_SAVE_OBJ                    1138
#define IDC_MAP_VIEW_ZOOM               1138
#define IDS_MOVIE_PLAY                  1138
#define IDC_VIEWER                      1140
#define IDC_ADDRESSES                   1141
#define IDC_GO                          1143
#define IDC_8_BIT                       1144
#define IDC_16_BIT                      1145
#define IDC_32_BIT                      1146
#define IDC_OAM_VIEW                    1147
#define IDC_OAM_VIEW_ZOOM               1148
#define IDC_SPRITE                      1150
#define IDC_POS                         1151
#define IDC_MODE                        1152
#define IDC_COLORS                      1153
#define IDC_MAPBASE                     1153
#define IDC_PALETTE                     1154
#define IDC_CHARBASE                    1154
#define IDC_TILE                        1155
#define IDC_DIM                         1155
#define IDC_PRIO                        1156
#define IDC_NUMCOLORS                   1156
#define IDC_SCROLLBAR                   1157
#define IDC_PRIORITY                    1157
#define IDC_MOSAIC                      1158
#define IDC_SIZE2                       1159
#define IDC_OVERFLOW                    1159
#define IDC_ROT                         1160
#define IDC_FLAGS                       1161
#define IDC_COMMANDS                    1162
#define IDC_BANK                        1162
#define IDC_CURRENTS                    1163
#define IDC_ASSIGN                      1164
#define IDC_RESET                       1165
#define IDC_EDIT_KEY                    1166
#define IDC_ALREADY_AFFECTED            1167
#define IDC_TILE_VIEW                   1168
#define IDC_16_COLORS                   1169
#define IDC_256_COLORS                  1170
#define IDC_CHARBASE_0                  1173
#define IDC_CHARBASE_1                  1174
#define IDC_CHARBASE_2                  1175
#define IDC_CHARBASE_3                  1176
#define IDC_PALETTE_SLIDER              1177
#define IDC_CHARBASE_4                  1178
#define IDC_COLOR_BG0                   1178
#define IDC_COLOR_BG1                   1179
#define IDC_URL                         1179
#define IDC_COLOR_BG2                   1180
#define IDC_STRETCH                     1180
#define IDC_COLOR_BG3                   1181
#define IDC_COLOR_OB0                   1182
#define IDC_COLOR_OB1                   1183
#define IDC_COLOR_OB2                   1184
#define IDC_COLOR_OB3                   1185
#define IDC_TRANSLATOR_URL              1186
#define IDC_STATIC1                     1187
#define IDC_STATIC2                     1188
#define IDC_STATIC3                     1189
#define IDC_STATIC4                     1190
#define IDC_DEFAULT                     1191
#define IDC_USER1                       1192
#define IDC_USER2                       1193
#define IDC_DISASSEMBLE                 1196
#define IDC_AUTOMATIC                   1199
#define IDC_ARM                         1200
#define IDC_THUMB                       1201
#define IDC_AUTO_UPDATE                 1204
#define IDC_N                           1210
#define IDC_Z                           1211
#define IDC_C                           1212
#define IDC_V                           1213
#define IDC_F                           1214
#define IDC_I                           1215
#define IDC_T                           1216
#define IDC_PORT                        1217
#define IDC_VSCROLL                     1218
#define IDC_VERSION                     1219
#define IDC_VERBOSE_SWI                 1223
#define IDC_VERBOSE_UNALIGNED_ACCESS    1224
#define IDC_VERBOSE_ILLEGAL_WRITE       1225
#define IDC_VERBOSE_ILLEGAL_READ        1226
#define IDC_LOG                         1227
#define IDC_CLEAR                       1228
#define IDC_VERBOSE_DMA0                1229
#define IDC_VERBOSE_DMA1                1230
#define IDC_TILE_NUMBER                 1230
#define IDC_VERBOSE_DMA2                1231
#define IDC_XY                          1231
#define IDC_VERBOSE_DMA3                1232
#define IDC_VERBOSE_UNDEFINED           1233
#define IDC_TITLE                       1234
#define IDC_VERBOSE_AGBPRINT            1234
#define IDC_CURRENT_ADDRESS             1235
#define IDC_NOTES                       1236
#define IDC_CURRENT_ADDRESS_LABEL       1236
#define IDC_LOAD                        1238
#define IDC_SIZE_CONTROL                1240
#define IDC_MODES                       1240
#define IDC_DRIVERS                     1241
#define IDC_THROTTLE                    1242
#define IDC_H                           1243
#define IDC_OAP                         1244
#define IDC_BANK_0                      1245
#define IDC_BANK_1                      1246
#define IDC_TIMER                       1247
#define IDC_INTERVAL                    1248
#define IDC_BIT_0                       1250
#define IDC_BIT_1                       1251
#define IDC_PREDEFINED                  1251
#define IDC_BIT_2                       1252
#define IDC_BUG_REPORT                  1252
#define IDC_BIT_3                       1253
#define IDC_COPY                        1253
#define IDC_BIT_4                       1254
#define IDC_NAME                        1254
#define IDC_BIT_5                       1255
#define IDC_RTC                         1255
#define IDC_BIT_6                       1256
#define IDC_SAVE_TYPE                   1256
#define IDC_BIT_7                       1257
#define IDC_FLASH_SIZE                  1257
#define IDC_BIT_8                       1258
#define IDC_COMMENT                     1258
#define IDC_BIT_9                       1259
#define IDC_BIT_10                      1260
#define IDC_BIT_11                      1261
#define IDC_BIT_12                      1262
#define IDC_BIT_13                      1263
#define IDC_BIT_14                      1264
#define IDC_BIT_15                      1265
#define IDC_MIRRORING                   1266
#define IDC_LY                          1267
#define ID_HELP_ABOUT                   40001
#define ID_FILE_EXIT                    40002
#define ID_OPTIONS_VIDEO_FRAMESKIP_0    40003
#define ID_OPTIONS_VIDEO_FRAMESKIP_1    40004
#define ID_OPTIONS_VIDEO_FRAMESKIP_2    40005
#define ID_OPTIONS_VIDEO_FRAMESKIP_3    40006
#define ID_OPTIONS_VIDEO_FRAMESKIP_4    40007
#define ID_OPTIONS_VIDEO_FRAMESKIP_5    40008
#define ID_OPTIONS_VIDEO_VSYNC          40009
#define ID_OPTIONS_VIDEO_X1             40010
#define ID_OPTIONS_VIDEO_X2             40011
#define ID_OPTIONS_VIDEO_X3             40012
#define ID_OPTIONS_VIDEO_X4             40013
#define ID_FILE_PAUSE                   40014
#define ID_OPTIONS_EMULATOR_DIRECTORIES 40015
#define ID_OPTIONS_JOYPAD               40016
#define ID_OPTIONS_EMULATOR_SYNCHRONIZE 40017
#define ID_FILE_RESET                   40018
#define ID_FILE_LOAD                    40019
#define ID_OPTIONS_SOUND_DIRECTSOUNDA   40020
#define ID_OPTIONS_SOUND_DIRECTSOUNDB   40021
#define ID_OPTIONS_SOUND_OFF            40022
#define ID_OPTIONS_SOUND_MUTE           40023
#define ID_OPTIONS_SOUND_ON             40024
#define ID_OPTIONS_SOUND_CHANNEL1       40025
#define ID_OPTIONS_SOUND_CHANNEL2       40026
#define ID_OPTIONS_SOUND_CHANNEL3       40027
#define ID_OPTIONS_SOUND_CHANNEL4       40028
#define ID_OPTIONS_EMULATOR_USEBIOSFILE 40029
#define ID_OPTIONS_EMULATOR_SELECTBIOSFILE 40030
#define ID_CHEATS_SEARCHFORCHEATS       40031
#define ID_CHEATS_ADDCHEAT              40032
#define ID_OPTIONS_VIDEO_DISABLESFX     40033
#define ID_OPTIONS_GAMEBOY_BORDER       40034
#define ID_FILE_SAVEGAME_SLOT1          40035
#define ID_FILE_SAVEGAME_SLOT2          40036
#define ID_FILE_SAVEGAME_SLOT3          40037
#define ID_FILE_SAVEGAME_SLOT4          40038
#define ID_FILE_SAVEGAME_SLOT5          40039
#define ID_FILE_SAVEGAME_SLOT6          40040
#define ID_FILE_SAVEGAME_SLOT7          40041
#define ID_FILE_SAVEGAME_SLOT8          40042
#define ID_FILE_SAVEGAME_SLOT9          40043
#define ID_FILE_SAVEGAME_SLOT10         40044
#define ID_FILE_LOADGAME_SLOT1          40045
#define ID_FILE_LOADGAME_SLOT2          40046
#define ID_FILE_LOADGAME_SLOT3          40047
#define ID_FILE_LOADGAME_SLOT4          40048
#define ID_FILE_LOADGAME_SLOT5          40049
#define ID_FILE_LOADGAME_SLOT6          40050
#define ID_FILE_LOADGAME_SLOT7          40051
#define ID_FILE_LOADGAME_SLOT8          40052
#define ID_FILE_LOADGAME_SLOT9          40053
#define ID_FILE_LOADGAME_SLOT10         40054
#define ID_OPTIONS_GAMEBOY_AUTOMATIC    40057
#define ID_OPTIONS_GAMEBOY_CGB          40058
#define ID_OPTIONS_GAMEBOY_GBA          40059
#define ID_OPTIONS_GAMEBOY_SGB          40060
#define ID_OPTIONS_GAMEBOY_GB           40062
#define ID_OPTIONS_GAMEBOY_REALCOLORS   40063
#define ID_OPTIONS_GAMEBOY_GAMEBOYCOLORS 40064
#define ID_CHEATS_GAMEBOY               40065
#define ID_OPTIONS_SOUND_11KHZ          40067
#define ID_OPTIONS_SOUND_22KHZ          40068
#define ID_OPTIONS_SOUND_44KHZ          40069
#define ID_OPTIONS_VIDEO_DDRAWEMULATIONONLY 40070
#define ID_OPTIONS_VIDEO_DDRAWUSEVIDEOMEMORY 40071
#define ID_OPTIONS_PRIORITY_HIGHEST     40072
#define ID_OPTIONS_PRIORITY_ABOVENORMAL 40073
#define ID_OPTIONS_PRIORITY_NORMAL      40074
#define ID_OPTIONS_PRIORITY_BELOWNORMAL 40075
#define ID_OPTIONS_VIDEO_FULLSCREEN320X240 40076
#define ID_OPTIONS_VIDEO_FULLSCREEN640X480 40077
#define ID_OPTIONS_FILTER_NORMAL        40078
#define ID_OPTIONS_FILTER_2XSAI         40079
#define ID_OPTIONS_FILTER_SUPER2XSAI    40081
#define ID_OPTIONS_FILTER_SUPEREAGLE    40082
#define ID_OPTIONS_FILTER_TVMODE        40083
#define ID_CHEATS_CHEATLIST             40084
#define ID_OPTIONS_JOYPAD_AUTOFIRE_A    40085
#define ID_OPTIONS_JOYPAD_AUTOFIRE_B    40086
#define ID_OPTIONS_JOYPAD_AUTOFIRE_L    40087
#define ID_OPTIONS_JOYPAD_AUTOFIRE_R    40088
#define ID_OPTIONS_VIDEO_FULLSCREENSTRETCHTOFIT 40089
#define ID_OPTIONS_EMULATOR_ASSOCIATE   40091
#define ID_OPTIONS_FILTER_DISABLEMMX    40093
#define ID_OPTIONS_1X                   40096
#define ID_OPTIONS_2X                   40097
#define ID_OPTIONS_3X                   40098
#define ID_OPTIONS_4X                   40099
#define ID_FILE_ROMINFORMATION          40100
#define ID_CHEATS_ADDCHEATCODE          40101
#define ID_OPTIONS_EMULATOR_DISABLESTATUSMESSAGES 40102
#define ID_OPTIONS_JOYPAD_MOTIONCONFIGURE 40103
#define ID_FILE_SCREENCAPTURE           40104
#define ID_OPTIONS_LANGUAGE_SYSTEM      40105
#define ID_OPTIONS_LANGUAGE_ENGLISH     40106
#define ID_OPTIONS_LANGUAGE_OTHER       40107
#define ID_OPTIONS_GAMEBOY_PRINTER      40108
#define ID_FILE_RECENT_RESET            40109
#define ID_CHEATS_SAVECHEATLIST         40110
#define ID_CHEATS_LOADCHEATLIST         40111
#define ID_CHEATS_AUTOMATICSAVELOADCHEATS 40112
#define ID_FILE_IMPORT_GAMESHARKSNAPSHOT 40115
#define ID_FILE_IMPORT_BATTERYFILE      40116
#define ID_FILE_IMPORT_GAMESHARKCODEFILE 40117
#define ID_FILE_EXPORT_BATTERYFILE      40118
#define ID_OPTIONS_FILTER16BIT_PIXELATEEXPERIMENTAL 40121
#define ID_OPTIONS_FILTER16BIT_MOTIONBLUREXPERIMENTAL 40122
#define ID_OPTIONS_EMULATOR_PAUSEWHENINACTIVE 40124
#define ID_OPTIONS_SOUND_STARTRECORDING 40125
#define ID_OPTIONS_SOUND_STOPRECORDING  40126
#define ID_OPTIONS_VIDEO_LAYERS_BG0     40127
#define ID_OPTIONS_VIDEO_LAYERS_BG1     40128
#define ID_OPTIONS_VIDEO_LAYERS_BG2     40129
#define ID_OPTIONS_VIDEO_LAYERS_BG3     40130
#define ID_OPTIONS_VIDEO_LAYERS_OBJ     40131
#define ID_OPTIONS_VIDEO_LAYERS_WIN0    40132
#define ID_OPTIONS_VIDEO_LAYERS_WIN1    40133
#define ID_OPTIONS_VIDEO_LAYERS_OBJWIN  40134
#define ID_FILE_OPENGAMEBOY             40135
#define ID_OPTIONS_SOUND_USEOLDSYNCHRONIZATION 40136
#define ID_DEBUG_NEXTFRAME              40137
#define ID_TOOLS_MAPVIEW                40138
#define ID_TOOLS_PALETTEVIEW            40139
#define ID_OPTIONS_EMULATOR_PNGFORMAT   40140
#define ID_OPTIONS_EMULATOR_BMPFORMAT   40141
#define ID_TOOLS_MEMORYVIEWER           40143
#define ID_TOOLS_OAMVIEWER              40144
#define ID_TOOLS_CUSTOMIZE              40145
#define ID_TOOLS_TILEVIEWER             40146
#define ID_OPTIONS_GAMEBOY_COLORS       40147
#define ID_OPTIONS_SOUND_ECHO           40148
#define ID_OPTIONS_SOUND_LOWPASSFILTER  40149
#define ID_OPTIONS_SOUND_REVERSESTEREO  40150
#define ID_TOOLS_DISASSEMBLE            40151
#define ID_TOOLS_DEBUG_GDB              40152
#define ID_TOOLS_DEBUG_LOADANDWAIT      40153
#define ID_TOOLS_DEBUG_DISCONNECT       40154
#define ID_TOOLS_DEBUG_BREAK            40155
#define ID_TOOLS_LOGGING                40156
#define ID_OPTIONS_EMULATOR_SPEEDHACK   40157
#define ID_OPTIONS_EMULATOR_SPEEDUPTOGGLE 40158
#define ID_OPTIONS_FILTER16BIT_ADVANCEMAMESCALE2X 40160
#define ID_OPTIONS_FILTER16BIT_SIMPLE2X 40161
#define ID_FILE_RECENT_FREEZE           40162
#define ID_FILE_EXPORT_GAMESHARKSNAPSHOT 40163
#define ID_OPTIONS_VIDEO_FULLSCREEN800X600 40164
#define ID_OPTIONS_VIDEO_FRAMESKIP_6    40165
#define ID_OPTIONS_VIDEO_FRAMESKIP_7    40166
#define ID_OPTIONS_VIDEO_FRAMESKIP_8    40167
#define ID_OPTIONS_VIDEO_FRAMESKIP_9    40168
#define ID_OPTIONS_EMULATOR_SAVETYPE_AUTOMATIC 40169
#define ID_OPTIONS_EMULATOR_SAVETYPE_EEPROM 40170
#define ID_OPTIONS_EMULATOR_SAVETYPE_SRAM 40171
#define ID_OPTIONS_EMULATOR_SAVETYPE_FLASH 40172
#define ID_OPTIONS_EMULATOR_SAVETYPE_EEPROMSENSOR 40173
#define ID_OPTIONS_EMULATOR_SAVETYPE_FLASH512K 40174
#define ID_OPTIONS_EMULATOR_SAVETYPE_FLASH1M 40175
#define ID_OPTIONS_EMULATOR_AUTOMATICALLYIPSPATCH 40176
#define ID_TOOLS_RECORD_STARTAVIRECORDING 40178
#define ID_TOOLS_RECORD_STOPAVIRECORDING 40179
#define ID_OPTIONS_SOUND_VOLUME_1X      40182
#define ID_OPTIONS_SOUND_VOLUME_2X      40183
#define ID_OPTIONS_SOUND_VOLUME_3X      40184
#define ID_OPTIONS_SOUND_VOLUME_4X      40185
#define ID_OPTIONS_FILTER_BILINEAR      40186
#define ID_OPTIONS_FILTER_BILINEARPLUS  40187
#define ID_OPTIONS_FILTER_INTERFRAMEBLENDING_NONE 40188
#define ID_OPTIONS_FILTER_INTERFRAMEBLENDING_MOTIONBLUR 40189
#define ID_OPTIONS_FILTER_INTERFRAMEBLENDING_SMART 40190
#define ID_OPTIONS_VIDEO_FULLSCREEN     40191
#define ID_OPTIONS_VIDEO_TRIPLEBUFFERING 40192
#define ID_OPTIONS_FRAMESKIP_AUTOMATIC  40194
#define ID_OPTIONS_EMULATOR_SHOWSPEED_NONE 40195
#define ID_OPTIONS_EMULATOR_SHOWSPEED_PERCENTAGE 40196
#define ID_OPTIONS_EMULATOR_SHOWSPEED_DETAILED 40197
#define ID_OPTIONS_EMULATOR_SHOWSPEED_TRANSPARENT 40198
#define ID_OPTIONS_JOYPAD_CONFIGURE_1   40199
#define ID_OPTIONS_JOYPAD_CONFIGURE_2   40200
#define ID_OPTIONS_JOYPAD_CONFIGURE_3   40201
#define ID_OPTIONS_JOYPAD_CONFIGURE_4   40202
#define ID_OPTIONS_JOYPAD_DEFAULTJOYPAD_1 40208
#define ID_OPTIONS_JOYPAD_DEFAULTJOYPAD_2 40209
#define ID_OPTIONS_JOYPAD_DEFAULTJOYPAD_3 40210
#define ID_OPTIONS_JOYPAD_DEFAULTJOYPAD_4 40211
#define ID_OPTIONS_EMULATOR_STORESETTINGSINREGISTRY 40214
#define ID_FILE_EXPORT_SETTINGSTOINI    40215
#define ID_OPTIONS_FRAMESKIP_THROTTLE_NOTHROTTLE 40216
#define ID_OPTIONS_FRAMESKIP_THROTTLE_50 40217
#define ID_OPTIONS_FRAMESKIP_THROTTLE_150 40218
#define ID_OPTIONS_FRAMESKIP_THROTTLE_200 40219
#define ID_OPTIONS_FRAMESKIP_THROTTLE_25 40220
#define ID_OPTIONS_FRAMESKIP_THROTTLE_OTHER 40221
#define ID_OPTIONS_FRAMESKIP_THROTTLE_100 40222
#define ID_OPTIONS_FILTER_SCANLINES     40223
#define ID_OPTIONS_VIDEO_RENDERMETHOD_GDI 40228
#define ID_OPTIONS_VIDEO_RENDERMETHOD_DIRECTDRAW 40229
#define ID_OPTIONS_VIDEO_RENDERMETHOD_DIRECT3D 40230
#define ID_OPTIONS_VIDEO_RENDERMETHOD_OPENGL 40231
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_D3DNOFILTER 40233
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_D3DBILINEAR 40234
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_D3DTRILINEAR 40235
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_D3DANISOTROPIC 40236
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_GLNEAREST 40237
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_GLBILINEAR 40238
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_GLTRIANGLE 40239
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_GLQUADS 40240
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_SELECTSKIN 40245
#define ID_OPTIONS_VIDEO_RENDEROPTIONS_SKIN 40246
#define ID_OPTIONS_EMULATOR_AGBPRINT    40247
#define ID_OPTIONS_EMULATOR_REALTIMECLOCK 40248
#define ID_OPTIONS_GAMEBOY_SGB2         40249
#define ID_SYSTEM_MINIMIZE              40250
#define ID_TOOLS_RECORD_STARTMOVIERECORDING 40251
#define ID_TOOLS_RECORD_STOPMOVIERECORDING 40252
#define ID_TOOLS_PLAY_STARTMOVIEPLAYING 40253
#define ID_TOOLS_PLAY_STOPMOVIEPLAYING  40254
#define ID_OPTIONS_EMULATOR_AUTOHIDEMENU 40255
#define ID_OPTIONS_GAMEBOY_BORDERAUTOMATIC 40256
#define ID_OPTIONS_EMULATOR_REWIND      40257
#define ID_TOOLS_REWIND                 40258
#define ID_OPTIONS_EMULATOR_SKIPBIOS    40259
#define ID_HELP_BUGREPORT               40260
#define ID_HELP_FAQ                     40261
#define ID_OPTIONS_EMULATOR_REWINDINTERVAL 40262
#define ID_FILE_TOGGLEMENU              40263
#define ID_OPTIONS_EMULATOR_SAVETYPE_NONE 40264
#define ID_TOOLS_IOVIEWER               40266
#define ID_FILE_LOADGAME_MOSTRECENT     40267
#define ID_FILE_SAVEGAME_OLDESTSLOT     40268
#define ID_FILE_LOADGAME_AUTOLOADMOSTRECENT 40269
#define ID_OPTIONS_SOUND_VOLUME_5X      40270
#define ID_OPTIONS_SOUND_VOLUME_25X     40271
#define ID_CHEATS_DISABLECHEATS         40272
#define ID_OPTIONS_VIDEO_FULLSCREENMAXSCALE 40273
#define ID_OPTIONS_FILTER_HQ2X          40274
#define ID_OPTIONS_FILTER_LQ2X          40275
#define ID_OPTIONS_EMULATOR_GAMEOVERRIDES 40276
#define ID_HELP_GNUPUBLICLICENSE        40277
#define ID_OPTIONS_EMULATOR_GENERICFLASHCARD 40279
#define ID_OPTIONS_SOUND_HARDWAREACCELERATION 40281

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        158
#define _APS_NEXT_COMMAND_VALUE         40282
#define _APS_NEXT_CONTROL_VALUE         1256
#define _APS_NEXT_SYMED_VALUE           108
#endif
#endif
