#ifndef YYSTYPE
#define YYSTYPE int
#endif
#define TOKEN_IDENTIFIER        257
#define TOKEN_DOT       258
#define TOKEN_STAR      259
#define TOKEN_ARROW     260
#define TOKEN_ADDR      261
#define TOKEN_SIZEOF    262
#define TOKEN_NUMBER    263


extern YYSTYPE yylval;
